@@ -1,7 +1,7 @@
 /*
  * This file is part of LaTeXila.
  *
- * Copyright © 2009 Sébastien Wilmet
+ * Copyright © 2009, 2010 Sébastien Wilmet
  *
  * LaTeXila is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,7 @@
  */
 
 #include <stdlib.h>
-#include <stdio.h>
-#include <locale.h>
-#include <libintl.h>
 #include <gtk/gtk.h>
-#include <gtksourceview/gtksourceview.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagemanager.h>
 
 #include "main.h"
 #include "config.h"
@@ -35,6 +29,8 @@
 #include "prefs.h"
 #include "ui.h"
 #include "templates.h"
+#include "latex_output_filter.h"
+#include "log.h"
 
 static gboolean option_version (const gchar *option_name, const gchar *value,
 		gpointer data, GError **error);
@@ -44,7 +40,6 @@ static void init_source_view (GtkWidget *vbox_source_view);
 static void init_go_to_line (GtkWidget *vbox_source_view);
 static void init_find (GtkWidget *vbox_source_view);
 static void init_replace (GtkWidget *vbox_source_view);
-static void init_log_zone (void);
 static void init_statusbar (GtkWidget *main_vbox);
 
 latexila_t latexila = {NULL};
@@ -91,6 +86,41 @@ init_main_window (void)
 	gtk_window_set_default_size (GTK_WINDOW (window),
 			latexila.prefs.window_width, latexila.prefs.window_height);
 
+	// icons
+	GList *list_icons = NULL;
+	GError *error = NULL;
+	gchar *filenames[] =
+	{
+		DATA_DIR "/images/logo/icon16.png",
+		DATA_DIR "/images/logo/icon22.png",
+		DATA_DIR "/images/logo/icon24.png",
+		DATA_DIR "/images/logo/icon32.png",
+		DATA_DIR "/images/logo/icon48.png",
+	}; 
+
+	guint nb_icons = G_N_ELEMENTS (filenames);
+
+	for (int i = 0 ; i < nb_icons ; i++)
+	{
+		// TODO check memory leaks here
+		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filenames[i], &error);
+		if (error != NULL)
+		{
+			print_warning ("%s", error->message);
+			g_error_free (error);
+			error = NULL;
+			continue;
+		}
+
+		list_icons = g_list_append (list_icons, pixbuf);
+	}
+
+	gtk_window_set_default_icon_list (list_icons);
+
+	// Gtk have a copy of the GList and adds a ref to every icon
+	g_list_foreach (list_icons, (GFunc) g_object_unref, NULL);
+	g_list_free (list_icons);
+
 	if (latexila.prefs.window_maximised)
 		gtk_window_maximize (GTK_WINDOW (window));
 }
@@ -104,35 +134,38 @@ init_side_pane (void)
 	gtk_paned_add1 (latexila.main_hpaned, side_pane_notebook);
 
 	// symbol tables
-	GtkWidget *vbox_symbols = gtk_vbox_new (FALSE, 0);
-	latexila.symbols.vbox = vbox_symbols;
+	{
+		GtkWidget *vbox_symbols = gtk_vbox_new (FALSE, 0);
+		latexila.symbols.vbox = vbox_symbols;
 
-	GtkWidget *tab_label = gtk_hbox_new (FALSE, 3);
-	GtkWidget *label = gtk_label_new (_("Symbols"));
-	GtkWidget *image = gtk_image_new_from_file (DATA_DIR "/images/greek/01.png");
-	gtk_box_pack_start (GTK_BOX (tab_label), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (tab_label), label, TRUE, TRUE, 0);
-	gtk_widget_show_all (tab_label);
+		GtkWidget *tab_label = gtk_hbox_new (FALSE, 3);
+		GtkWidget *label = gtk_label_new (_("Symbols"));
+		GtkWidget *image = gtk_image_new_from_file (DATA_DIR "/images/greek/01.png");
+		gtk_box_pack_start (GTK_BOX (tab_label), image, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (tab_label), label, TRUE, TRUE, 0);
+		gtk_widget_show_all (tab_label);
 
-	gtk_notebook_append_page (GTK_NOTEBOOK (side_pane_notebook), vbox_symbols,
-			tab_label);
-	init_symbols ();
+		gtk_notebook_append_page (GTK_NOTEBOOK (side_pane_notebook), vbox_symbols,
+				tab_label);
+		init_symbols ();
+	}
 
 	// file browser
-	GtkWidget *vbox_file_browser = gtk_vbox_new (FALSE, 0);
-	latexila.file_browser.vbox = vbox_file_browser;
+	{
+		GtkWidget *vbox_file_browser = gtk_vbox_new (FALSE, 0);
 
-	tab_label = gtk_hbox_new (FALSE, 3);
-	label = gtk_label_new (_("File Browser"));
-	image = gtk_image_new_from_stock (GTK_STOCK_OPEN,
-			GTK_ICON_SIZE_MENU);
-	gtk_box_pack_start (GTK_BOX (tab_label), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (tab_label), label, TRUE, TRUE, 0);
-	gtk_widget_show_all (tab_label);
+		GtkWidget *tab_label = gtk_hbox_new (FALSE, 3);
+		GtkWidget *label = gtk_label_new (_("File Browser"));
+		GtkWidget *image = gtk_image_new_from_stock (GTK_STOCK_OPEN,
+				GTK_ICON_SIZE_MENU);
+		gtk_box_pack_start (GTK_BOX (tab_label), image, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (tab_label), label, TRUE, TRUE, 0);
+		gtk_widget_show_all (tab_label);
 
-	gtk_notebook_append_page (GTK_NOTEBOOK (side_pane_notebook),
-			vbox_file_browser, tab_label);
-	init_file_browser ();
+		gtk_notebook_append_page (GTK_NOTEBOOK (side_pane_notebook),
+				vbox_file_browser, tab_label);
+		init_file_browser (vbox_file_browser);
+	}
 }
 
 static void
@@ -294,64 +327,6 @@ init_replace (GtkWidget *vbox_source_view)
 	gtk_box_pack_start (GTK_BOX (hbox2), button_find, FALSE, FALSE, 0);
 }
 
-static void
-init_log_zone (void)
-{
-	// action history
-	GtkListStore *list_store = gtk_list_store_new (N_COLUMNS_ACTION,
-			G_TYPE_STRING, G_TYPE_POINTER);
-	latexila.action_log.list_store = list_store;
-	
-	GtkWidget *list_view = gtk_tree_view_new_with_model (
-			GTK_TREE_MODEL (list_store));
-	latexila.action_log.list_view = GTK_TREE_VIEW (list_view);
-	GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
-	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes (
-			_("Action history"), renderer, "text", COLUMN_ACTION_TITLE, NULL);	
-	gtk_tree_view_append_column (latexila.action_log.list_view, column);
-	
-	GtkTreeSelection *select =
-		gtk_tree_view_get_selection (latexila.action_log.list_view);
-	latexila.action_log.list_selection = select;
-	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
-	g_signal_connect (G_OBJECT (select), "changed",
-			G_CALLBACK (cb_action_list_changed), NULL);
-
-	// with a scrollbar
-	GtkWidget *scrollbar = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollbar),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_paned_add1 (GTK_PANED (latexila.log_hpaned), scrollbar);
-	gtk_container_add (GTK_CONTAINER (scrollbar), list_view);
-	
-	// log details
-	GtkWidget *log_view = gtk_text_view_new ();
-	GtkTextBuffer *log_buffer = gtk_text_view_get_buffer (
-			GTK_TEXT_VIEW (log_view));
-
-	latexila.action_log.text_view = GTK_TEXT_VIEW (log_view);
-	latexila.action_log.text_buffer = log_buffer;
-
-	gtk_text_buffer_set_text (log_buffer, _("Welcome to LaTeXila!"), -1);
-	gtk_text_view_set_editable (GTK_TEXT_VIEW (log_view), FALSE);
-	
-	// with a scrollbar
-	scrollbar = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollbar),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_paned_add2 (GTK_PANED (latexila.log_hpaned), scrollbar);
-	gtk_container_add (GTK_CONTAINER (scrollbar), log_view);
-
-	// tags
-	gtk_text_buffer_create_tag (log_buffer, "bold",
-			"weight", PANGO_WEIGHT_BOLD,
-			NULL);
-	gtk_text_buffer_create_tag (log_buffer, "error",
-			"foreground", "red",
-			NULL);
-	latexila.action_log.tag_table = gtk_text_buffer_get_tag_table (log_buffer);
-}
-
 static void
 init_statusbar (GtkWidget *main_vbox)
 {
@@ -373,14 +348,20 @@ main (int argc, char *argv[])
 {
 	GError *error = NULL;
 
-	/* command line options */
-	gboolean option_new_document = FALSE;
-
+	/* localisation */
 	gchar *latexila_nls_package = NULL;
+
 #ifdef LATEXILA_NLS_ENABLED
+	setlocale (LC_ALL, "");
+	bindtextdomain (LATEXILA_NLS_PACKAGE, LATEXILA_NLS_LOCALEDIR);
+	bind_textdomain_codeset (LATEXILA_NLS_PACKAGE, "UTF-8");
+	textdomain (LATEXILA_NLS_PACKAGE);
 	latexila_nls_package = LATEXILA_NLS_PACKAGE;
 #endif
 
+	/* command line options */
+	gboolean option_new_document = FALSE;
+
 	GOptionEntry options[] = {
 		{ "version", 'v', G_OPTION_FLAG_IN_MAIN | G_OPTION_FLAG_NO_ARG,
 			G_OPTION_ARG_CALLBACK, (gpointer) option_version,
@@ -391,29 +372,29 @@ main (int argc, char *argv[])
 		{NULL}
 	};
 
-	GOptionContext *context = g_option_context_new ("[FILES]");
-	g_option_context_add_main_entries (context, options, latexila_nls_package);
+	GOptionContext *context = g_option_context_new (NULL);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_main_entries (context, options, latexila_nls_package);
 
 	// TODO with valgrind there are a lot of memory leaks with this, can we do
 	// something?
-	if (! g_option_context_parse (context, &argc, &argv, &error))
+	gboolean tmp = g_option_context_parse (context, &argc, &argv, &error);
+	g_option_context_free (context);
+	if (! tmp)
+	{
 		print_error ("option parsing failed: %s\n", error->message);
+		g_error_free (error);
+		exit (EXIT_FAILURE);
+	}
 
 	gtk_init_with_args (&argc, &argv, NULL, options, latexila_nls_package,
 			&error);
 	if (error != NULL)
+	{
 		print_error ("%s", error->message);
-
-	g_option_context_free (context);
-
-	/* localisation */
-#ifdef LATEXILA_NLS_ENABLED
-	setlocale (LC_ALL, "");
-	bindtextdomain (LATEXILA_NLS_PACKAGE, LATEXILA_NLS_LOCALEDIR);
-	bind_textdomain_codeset (LATEXILA_NLS_PACKAGE, "UTF-8");
-	textdomain (LATEXILA_NLS_PACKAGE);
-#endif
+		g_error_free (error);
+		exit (EXIT_FAILURE);
+	}
 
 	/* preferences */
 	load_preferences (&latexila.prefs);
@@ -439,7 +420,10 @@ main (int argc, char *argv[])
 	GtkWidget *main_vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (latexila.main_window), main_vbox);
 
-	init_ui (main_vbox);
+	// init_ui() will set log_toolbar so we can pass it to init_log_zone()
+	GtkWidget *log_toolbar;
+
+	init_ui (main_vbox, &log_toolbar);
 	
 	/* horizontal pane
 	 * left: side pane (symbol tables and the file browser)
@@ -481,10 +465,12 @@ main (int argc, char *argv[])
 	gtk_paned_set_position (GTK_PANED (hpaned), latexila.prefs.log_hpaned_pos);
 	gtk_paned_add2 (GTK_PANED (vpaned), hpaned);
 
-	init_log_zone ();
+	init_log_zone (latexila.log_hpaned, log_toolbar);
 
-	/* statusbar */
+	/* init various things */
 	init_statusbar (main_vbox);
+	init_templates ();
+	latex_output_filter_init ();
 
 	/* show the window */
 	gtk_widget_show_all (GTK_WIDGET (latexila.main_window));
@@ -504,14 +490,20 @@ main (int argc, char *argv[])
 
 	/* reopen files on startup */
 	gchar ** list_opened_docs = (gchar **) latexila.prefs.list_opened_docs->pdata;
-	for (int i = 0 ; i < latexila.prefs.list_opened_docs->len
-			&& latexila.prefs.reopen_files_on_startup ; i++)
+	for (int i = 0 ; i < latexila.prefs.list_opened_docs->len ; i++)
 	{
-		open_new_document_without_uri (list_opened_docs[i]);
+		if (latexila.prefs.reopen_files_on_startup)
+			open_new_document_without_uri (list_opened_docs[i]);
+
+		// in all cases we must free the string
+		g_free (list_opened_docs[i]);
 	}
 	g_ptr_array_free (latexila.prefs.list_opened_docs, TRUE);
 	latexila.prefs.list_opened_docs = g_ptr_array_new ();
 
+	// go to the first file
+	gtk_notebook_set_current_page (latexila.notebook, 0);
+
 	/* open documents given in arguments */
 	for (int i = 1 ; i < argc ; i++)
 	{
@@ -533,8 +525,13 @@ main (int argc, char *argv[])
 	if (option_new_document)
 		cb_new ();
 
-	/* templates */
-	init_templates ();
+	/* timeout for saving files every X minutes */
+	if (latexila.prefs.auto_save)
+		latexila.auto_save_timeout = g_timeout_add_seconds (
+				60 * latexila.prefs.auto_save_interval,
+				(GSourceFunc) auto_save_files, NULL);
+	else
+		latexila.auto_save_timeout = 0;
 
 
 	gtk_main ();
